--- okular/core/document.cpp.orig	2009-11-27 14:03:13.000000000 +0100
+++ okular/core/document.cpp	2009-12-27 03:56:30.000000000 +0100
@@ -16,6 +16,12 @@
 #include <windows.h>
 #endif
 
+#ifdef Q_OS_FREEBSD
+#include <sys/types.h>
+#include <sys/sysctl.h>
+#include <unistd.h>
+#endif
+
 // qt/kde/system includes
 #include <QtCore/QtAlgorithms>
 #include <QtCore/QDir>
@@ -248,6 +254,12 @@
         if ( entry.startsWith( "MemTotal:" ) )
             return (cachedValue = (Q_UINT64_C(1024) * entry.section( ' ', -2, -2 ).toULongLong()));
     }
+#elif defined(Q_OS_FREEBSD)
+    size_t total, len;
+
+    len = sizeof(total);
+    if (sysctlbyname("hw.physmem", &total, &len, NULL, 0) == 0)
+        return (cachedValue = total);
 #elif defined(Q_OS_WIN)
     MEMORYSTATUSEX stat;
     stat.dwLength = sizeof(stat);
@@ -294,6 +306,29 @@
     lastUpdate = QTime::currentTime();
 
     return ( cachedValue = (Q_UINT64_C(1024) * memoryFree) );
+#elif defined(Q_OS_FREEBSD)
+    // Consider free memory to be inactive + buffers + cached + free
+
+    size_t len, buffers = 0, inactive = 0, cached = 0, mfree = 0;
+    int pagesize = getpagesize();
+
+    len = sizeof(buffers);
+    if (sysctlbyname("vfs.bufspace", &buffers, &len, NULL, 0) == -1)
+        buffers = 0;
+
+    len = sizeof(inactive);
+    if (sysctlbyname("vm.stats.vm.v_inactive_count", &inactive, &len, NULL, 0) == -1)
+        inactive = 0;
+
+    len = sizeof(cached);
+    if (sysctlbyname("vm.stats.vm.v_cache_count", &cached, &len, NULL, 0) == -1)
+        cached = 0;
+
+    len = sizeof(mfree);
+    if (sysctlbyname("vm.stats.vm.v_free_count", &mfree, &len, NULL, 0) == -1)
+        mfree = 0;
+
+    return ( cachedValue = (inactive + cached + mfree) * pagesize + buffers );
 #elif defined(Q_OS_WIN)
     MEMORYSTATUSEX stat;
     stat.dwLength = sizeof(stat);
